@@ -1,5 +1,5 @@
 ##############################################################################
-# Copyright (c) 2011 Hajime Nakagami<nakagami@gmail.com>
+# Copyright (c) 2011-2013 Hajime Nakagami<nakagami@gmail.com>
 # All rights reserved.
 # Licensed under the New BSD License
 # (http://www.freebsd.org/copyright/freebsd-license.html)
@@ -136,6 +136,9 @@ SQL_TYPE_QUAD = 550
 SQL_TYPE_TIME = 560
 SQL_TYPE_DATE = 570
 SQL_TYPE_INT64 = 580
+SQL_TYPE_BOOLEAN = 32764
+SQL_TYPE_NULL = 32766
+
 
 ISOLATION_LEVEL_READ_UNCOMMITTED = 0
 ISOLATION_LEVEL_READ_COMMITED = 1
@@ -334,3 +337,4 @@ isc_arg_next_mach = 15
 isc_arg_netware = 16
 isc_arg_win32 = 17
 isc_arg_warning = 18
+isc_arg_sql_state = 19
