@@ -1,5 +1,5 @@
 ##############################################################################
-# Copyright (c) 2009-2012 Hajime Nakagami<nakagami@gmail.com>
+# Copyright (c) 2009-2013 Hajime Nakagami<nakagami@gmail.com>
 # All rights reserved.
 # Licensed under the New BSD License
 # (http://www.freebsd.org/copyright/freebsd-license.html)
@@ -171,6 +171,9 @@ def params_to_blr(params):
         elif t == datetime.datetime:
             v = convert_timestamp(p)
             blr += bytes([35])
+        elif t == bool:
+            v = bytes([1]) if p else bytes([0])
+            blr += bytes([23])
         elif p == None:
             v = bytes([0]) * 8
             blr += bytes([9, 0])
@@ -307,9 +310,12 @@ class WireProtocol:
                 message = message.replace('@' + str(num_arg), str(num))
             elif n == isc_arg_string or n == isc_arg_interpreted:
                 nbytes = bytes_to_bint(recv_channel(self.sock, 4))
-                n = str(recv_channel(self.sock, nbytes, word_alignment=True))
+                s = str(recv_channel(self.sock, nbytes, word_alignment=True))
                 num_arg += 1
-                message = message.replace('@' + str(num_arg), n)
+                message = message.replace('@' + str(num_arg), s)
+            elif n == isc_arg_sql_state:
+                nbytes = bytes_to_bint(recv_channel(self.sock, 4))
+                s = str(recv_channel(self.sock, nbytes, word_alignment=True))
             n = bytes_to_bint(recv_channel(self.sock, 4))
 
         return (gds_codes, sql_code, message)
@@ -515,16 +521,6 @@ class WireProtocol:
         p.pack_bytes(b)
         p.pack_int(self.buffer_length)
         send_channel(self.sock, p.get_buffer())
-    @wire_operation
-    def _op_info_database(self, b):
-        p = xdrlib.Packer()
-        p.pack_int(self.op_info_database)
-        p.pack_int(self.db_handle)
-        p.pack_int(0)
-        p.pack_bytes(b)
-        p.pack_int(self.buffer_length)
-        send_channel(self.sock, p.get_buffer())
-
 
     @wire_operation
     def _op_free_statement(self, stmt_handle, mode):
